package com.yiqi.framework.aspectj;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.yiqi.common.annotation.GmLog;
import com.yiqi.common.annotation.Log;
import com.yiqi.common.core.domain.AuthHolder;
import com.yiqi.common.core.domain.model.LoginUser;
import com.yiqi.common.enums.*;
import com.yiqi.common.filter.PropertyPreExcludeFilter;
import com.yiqi.common.utils.SecurityUtils;
import com.yiqi.common.utils.ServletUtils;
import com.yiqi.common.utils.StringUtils;
import com.yiqi.common.utils.ip.IpUtils;
import com.yiqi.framework.manager.AsyncManager;
import com.yiqi.framework.manager.factory.AsyncFactory;
import com.yiqi.system.domain.SysOperLog;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Map;

/**
 * 用于记录注解上接口的入参出参，统一规范。
 */
@Aspect
@Component
@Slf4j
public class GmLogAspect {

    /**
     * 排除敏感属性字段
     */
    public static final String[] EXCLUDE_PROPERTIES = {"password", "oldPassword", "newPassword", "confirmPassword"};


    /**
     * 方法二：拦截该注解标识的方法
     */
    @Pointcut("@annotation(com.yiqi.common.annotation.GmLog)")
    public void logPointCut() {
    }

    /**
     * 环绕通知
     *
     * @param pjd
     * @return
     * @throws Throwable
     */
    //    @Around("privilege()") // 第一种方式
    @Around("logPointCut()") // 第二种方式
    public Object arount(ProceedingJoinPoint pjd) throws Throwable {
        long startTime = System.currentTimeMillis();
        // 类名
        String className = pjd.getTarget().getClass().getName();
        // 获取执行的方法名称
        String methodName = pjd.getSignature().getName();

        // 1. 如果是使用的第二种方式，则判断该方法是否使用了改注解
        // 2. 如果是使用的第一种方式，直接注释即可。
        GmLog gmLog = this.getAnnotationLog(pjd);
        if (gmLog != null) {
            String value = gmLog.value();
            log.info("{}.{}()【{}】：===================", className, methodName, value);
        }

        Object[] args = pjd.getArgs();
        try {
            String params = JSONUtil.toJsonStr(args[0]);
            //打印请求参数参数
            log.info("{}.{}()【方法请求参数为】：{}", className, methodName, params);
            Map<?, ?> paramsMap = ServletUtils.getParamMap(ServletUtils.getRequest());
            if (!paramsMap.isEmpty()) {
                log.info("{}.{}()【GET方法请求参数为】：{}", JSONUtil.toJsonStr(paramsMap));
            }
        } catch (Exception e) {
            log.info("{}.{}()【方法请求参数打印失败】：{}", className, methodName, e);
        }
        // 执行目标方法
        Object result = pjd.proceed();
        // 打印返回结果
        try {
            String s = JSONUtil.toJsonStr(result);
            log.info("{}.{}()【方法返回结果为】：{}", className, methodName, s);
        } catch (Exception e) {
            log.info("{}.{}()【方法返回结果打印失败】：{}", className, methodName, e);
        }
        // 获取执行完的时间
        long time = System.currentTimeMillis() - startTime;
        log.info("{}.{}()【方法执行时长为】：{}{}", className, methodName, time, " ms");
        // TODO 这里可以考虑新加一个异步方法，保存信息到数据库，入参，出参，请求人，请求时间，ip信息等，如果有异常，还有异常信息。
        if (gmLog != null) {
            boolean save = gmLog.isSave();
            if (save) {
                String val = gmLog.value();
                // 调用异步保存数据库方法
                this.saveLog(pjd, gmLog, val);
            }
        }
        return result;
    }

    private void saveLog(ProceedingJoinPoint joinPoint, GmLog gmLog, String val) {

        if (PlatformType.USER.getCode().equals(AuthHolder.platform())) {
            return;
        }
        // 获取当前的用户
        LoginUser loginUser = SecurityUtils.getLoginUser();

        // *========数据库日志=========*//
        SysOperLog operLog = new SysOperLog();
        operLog.setStatus(BusinessStatus.SUCCESS.ordinal());
        // 请求的地址
        String ip = IpUtils.getIpAddr(ServletUtils.getRequest());
        operLog.setOperIp(ip);
        operLog.setOperUrl(StringUtils.substring(ServletUtils.getRequest().getRequestURI(), 0, 255));
        if (loginUser != null) {
            operLog.setOperName(loginUser.getUsername());
        }
        // 设置方法名称
        String className = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();
        operLog.setMethod(className + "." + methodName + "()");
        // 设置请求方式
        operLog.setRequestMethod(ServletUtils.getRequest().getMethod());
        // 处理设置注解上的参数
        getControllerMethodDescription(joinPoint, gmLog, operLog, null);
        // 保存数据库
        AsyncManager.me().execute(AsyncFactory.recordOper(operLog));

    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层注解
     *
     * @param log     日志
     * @param operLog 操作日志
     * @throws Exception
     */
    public void getControllerMethodDescription(JoinPoint joinPoint, GmLog log, SysOperLog operLog, Object jsonResult) {
        // 设置action动作
        operLog.setBusinessType(BusinessType.OTHER.ordinal());
        // 设置标题
        operLog.setTitle(log.value());
        // 设置操作人类别
        operLog.setOperatorType(OperatorType.MANAGE.ordinal());
        // 是否需要保存request，参数和值
        setRequestValue(joinPoint, operLog);
    }

    /**
     * 获取请求的参数，放到log中
     *
     * @param operLog 操作日志
     * @throws Exception 异常
     */
    private void setRequestValue(JoinPoint joinPoint, SysOperLog operLog) {
        String requestMethod = operLog.getRequestMethod();
        if (HttpMethod.PUT.name().equals(requestMethod) || HttpMethod.POST.name().equals(requestMethod)) {
            String params = argsArrayToString(joinPoint.getArgs());
            operLog.setOperParam(StringUtils.substring(params, 0, 2000));
        } else {
            Map<?, ?> paramsMap = ServletUtils.getParamMap(ServletUtils.getRequest());
            operLog.setOperParam(StringUtils.substring(JSON.toJSONString(paramsMap, excludePropertyPreFilter()), 0, 2000));
        }
    }

    /**
     * 参数拼装
     */
    private String argsArrayToString(Object[] paramsArray) {
        String params = "";
        if (paramsArray != null && paramsArray.length > 0) {
            for (Object o : paramsArray) {
                if (StringUtils.isNotNull(o) && !isFilterObject(o)) {
                    try {
                        String jsonObj = JSON.toJSONString(o, excludePropertyPreFilter());
                        params += jsonObj.toString() + " ";
                    } catch (Exception e) {
                    }
                }
            }
        }
        return params.trim();
    }

    /**
     * 忽略敏感属性
     */
    public PropertyPreExcludeFilter excludePropertyPreFilter() {
        return new PropertyPreExcludeFilter().addExcludes(EXCLUDE_PROPERTIES);
    }

    /**
     * 判断是否需要过滤的对象。
     *
     * @param o 对象信息。
     * @return 如果是需要过滤的对象，则返回true；否则返回false。
     */
    @SuppressWarnings("rawtypes")
    public boolean isFilterObject(final Object o) {
        Class<?> clazz = o.getClass();
        if (clazz.isArray()) {
            return clazz.getComponentType().isAssignableFrom(MultipartFile.class);
        } else if (Collection.class.isAssignableFrom(clazz)) {
            Collection collection = (Collection) o;
            for (Object value : collection) {
                return value instanceof MultipartFile;
            }
        } else if (Map.class.isAssignableFrom(clazz)) {
            Map map = (Map) o;
            for (Object value : map.entrySet()) {
                Map.Entry entry = (Map.Entry) value;
                return entry.getValue() instanceof MultipartFile;
            }
        }
        return o instanceof MultipartFile || o instanceof HttpServletRequest || o instanceof HttpServletResponse
                || o instanceof BindingResult;
    }

    /**
     * 是否存在注解，如果存在就获取
     *
     * @param joinPoint
     * @return
     */
    private GmLog getAnnotationLog(JoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();
        if (method != null) {
            return method.getAnnotation(GmLog.class);
        }
        return null;
    }

}
